@@ -7,8 +7,8 @@
 
 from __future__ import annotations
 
-import json
 import time
+import types
 import uuid
 from os import mkdir, path
 from shutil import copyfile, move
@@ -19,7 +19,7 @@ from geoh5py.workspace import Workspace
 from ipyfilechooser import FileChooser
 from ipywidgets import Button, Checkbox, HBox, Label, Text, ToggleButton, VBox, Widget
 
-from geoapps.io.params import Params
+from geoapps.io import InputFile, Params
 from geoapps.utils.formatters import string_name
 
 
@@ -47,7 +47,7 @@ class BaseApplication:
     plot_result = False
 
     def __init__(self, **kwargs):
-        self.defaults = self.update_defaults(**kwargs)
+        self.defaults.update(**kwargs)
         self._file_browser = FileChooser()
         self._file_browser._select.on_click(self.file_browser_change)
         self._file_browser._select.style = {"description_width": "initial"}
@@ -90,6 +90,11 @@ class BaseApplication:
         self.ga_group_name.observe(self.ga_group_name_update)
         self.__populate__(**self.defaults)
 
+        for key in list(self.__dict__.keys()):
+            attr = getattr(self, key, None)
+            if isinstance(attr, Widget) and hasattr(attr, "style"):
+                attr.style = {"description_width": "initial"}
+
     def __call__(self):
         return self.main
 
@@ -97,7 +102,6 @@ class BaseApplication:
         for key, value in kwargs.items():
             if key[0] == "_":
                 key = key[1:]
-
             if hasattr(self, "_" + key) or hasattr(self, key):
                 try:
                     if isinstance(getattr(self, key, None), Widget) and not isinstance(
@@ -108,7 +112,7 @@ class BaseApplication:
                                 value = [uuid.UUID(val) for val in value]
                             else:
                                 value = uuid.UUID(value)
-                        except (ValueError, AttributeError):
+                        except (ValueError, AttributeError, TypeError):
                             pass
 
                         widget = getattr(self, key)
@@ -120,21 +124,13 @@ class BaseApplication:
                         getattr(self, "_" + key, None), BaseApplication
                     ):
                         setattr(self, "_" + key, value)
+                    elif type(getattr(self, key, None)) is types.MethodType:
+                        getattr(self, key, None)(key, value)
                     else:
                         setattr(self, key, value)
                 except:
                     pass
 
-    def update_defaults(self, **kwargs):
-        """
-        Add defaults to the kwargs
-        """
-        for key, value in self.defaults.copy().items():
-            if key not in kwargs.keys():
-                kwargs[key] = value
-
-        return kwargs
-
     def file_browser_change(self, _):
         """
         Change the target h5file
@@ -143,10 +139,12 @@ class BaseApplication:
             _, extension = path.splitext(self.file_browser.selected)
 
             if extension == ".json" and getattr(self, "_param_class", None) is not None:
-                self.params = getattr(self, "_param_class").from_path(
-                    self.file_browser.selected
+                self.params = getattr(self, "_param_class")(
+                    InputFile(self.file_browser.selected)
                 )
-                self.__populate__(**self.params.__dict__)
+                self.refresh.value = False
+                self.__populate__(**self.params.to_dict(ui_json_format=False))
+                self.refresh.value = True
 
             elif extension == ".geoh5":
                 self.h5file = self.file_browser.selected
@@ -325,11 +323,6 @@ class BaseApplication:
         self._h5file = value
         self._workspace_geoh5 = value
         self._working_directory = None
-        self._file_browser.reset(
-            path=self.working_directory,
-            filename=path.basename(self._h5file),
-        )
-        self._file_browser._apply_selection()
         self.workspace = Workspace(self._h5file)
 
     @property
@@ -370,27 +363,6 @@ class BaseApplication:
             self._refresh = ToggleButton(value=False)
         return self._refresh
 
-    def save_json_params(self, file_name: str, out_dict: dict):
-        """"""
-        for key, params in out_dict.items():
-            if getattr(self, key, None) is not None:
-                value = getattr(self, key)
-                if hasattr(value, "value"):
-                    value = value.value
-                    if isinstance(value, uuid.UUID):
-                        value = str(value)
-
-                if isinstance(out_dict[key], dict):
-                    out_dict[key]["value"] = value
-                else:
-                    out_dict[key] = value
-
-        file = f"{path.join(self.working_directory, file_name)}.ui.json"
-        with open(file, "w") as f:
-            json.dump(out_dict, f, indent=4)
-
-        return out_dict
-
     @property
     def trigger(self) -> Button:
         """
@@ -420,8 +392,7 @@ class BaseApplication:
     @workspace.setter
     def workspace(self, workspace):
         assert isinstance(workspace, Workspace), f"Workspace must of class {Workspace}"
-        self._workspace = workspace
-        self._h5file = workspace.h5file
+        self.base_workspace_changes(workspace)
 
     @property
     def working_directory(self):
@@ -474,6 +445,22 @@ class BaseApplication:
     def run(cls, params):
         ...
 
+    def base_workspace_changes(self, workspace: Workspace):
+        self._workspace = workspace
+        self._h5file = workspace.h5file
+        self._file_browser.reset(
+            path=self.working_directory,
+            filename=path.basename(self._h5file),
+        )
+        self._file_browser._apply_selection()
+
+        export_path = path.join(path.abspath(path.dirname(self.h5file)), "Temp")
+        if not path.exists(export_path):
+            mkdir(export_path)
+
+        self.export_directory._set_form_values(export_path, "")
+        self.export_directory._apply_selection()
+
 
 def working_copy(h5file):
     """
