@@ -529,11 +529,13 @@ def inversion(input_file):
                     else:
                         topo = topo_entity.vertices
 
-                    if input_dict["topography"]["GA_object"]["data"] != "Z":
+                    try:
                         data = workspace.get_entity(
                             uuid.UUID(input_dict["topography"]["GA_object"]["data"])
                         )[0]
                         topo[:, 2] = data.values
+                    except (ValueError, IndexError):
+                        pass
 
                 if window is not None:
 
@@ -1547,9 +1549,9 @@ def inversion(input_file):
     #         vector=input_dict["inversion_type"][0:3] == 'mvi'
     #     )
     # )
-    # save_output = Directives.SaveOutputEveryIteration()
-    # save_output.fileName = workDir + "Output"
-    # directiveList.append(save_output)
+    save_output = Directives.SaveOutputEveryIteration()
+    save_output.fileName = workDir + "Output"
+    directiveList.append(save_output)
 
     # Put all the parts together
     inv = Inversion.BaseInversion(invProb, directiveList=directiveList)
