@@ -6,6 +6,8 @@
 #  (see LICENSE file at the root of this source code package).
 
 import sys
+import uuid
+from copy import deepcopy
 from os import path
 from typing import Optional
 
@@ -38,16 +40,17 @@ from ipywidgets import (
     interactive_output,
 )
 from ipywidgets.widgets.widget_selection import TraitError
+from tqdm import tqdm
 
 from geoapps.base import BaseApplication
+from geoapps.io import InputFile
+from geoapps.io.PeakFinder import PeakFinderParams
+from geoapps.io.PeakFinder.constants import app_initializer, default_ui_json
 from geoapps.selection import LineOptions, ObjectDataSelection
 from geoapps.utils import geophysical_systems
 from geoapps.utils.formatters import string_name
 from geoapps.utils.utils import LineDataDerivatives, hex_to_rgb, running_mean
 
-from ..io.PeakFinder import PeakFinderParams
-from ..io.PeakFinder.constants import default_ui_json
-
 _default_channel_groups = {
     "early": {"label": ["early"], "color": "#0000FF", "channels": []},
     "middle": {"label": ["middle"], "color": "#FFFF00", "channels": []},
@@ -75,27 +78,54 @@ class PeakFinder(ObjectDataSelection):
     Application for the picking of targets along Time-domain EM profiles
     """
 
+    defaults = {}
     _param_class = PeakFinderParams
     _add_groups = "only"
-    _object_types = (Curve,)
+    _center = None
+    _flip_sign = None
     _group_auto = None
+    _group_list = None
+    _group_display = None
+    _groups_setter = None
+    _lines = None
+    _markers = None
+    _max_migration = None
+    _min_amplitude = None
+    _min_channels = None
+    _min_value = None
+    _min_width = None
+    _plot_trigger = None
+    _residual = None
+    _scale_button = None
+    _scale_value = None
+    _show_decay = None
+    _smoothing = None
+    _structural_markers = None
+    _system = None
+    _tem_checkbox = None
+    _width = None
+    _object_types = (Curve,)
+    all_anomalies = []
+    active_channels = {}
+    _survey = None
+    _channel_groups = {}
+    pause_refresh = False
     decay_figure = None
     marker = {"left": "<", "right": ">"}
     plot_result = True
 
     def __init__(self, ui_json=None, **kwargs):
+        app_initializer.update(kwargs)
         if ui_json is not None and path.exists(ui_json):
-            self.params = self._param_class.from_path(ui_json)
+            self.params = self._param_class(InputFile(ui_json))
         else:
-            self.params = self._param_class.from_dict(
-                self._param_class._default_ui_json, **kwargs
-            )
-        self.defaults = self.update_defaults(**self.params.__dict__)
-        self.all_anomalies = []
-        self.active_channels = {}
-        self.pause_refresh = False
-        self._survey = None
-        self._channel_groups = {}
+            if "h5file" in app_initializer.keys():
+                app_initializer["geoh5"] = app_initializer.pop("h5file")
+
+            self.params = self._param_class(**app_initializer)
+
+        self.defaults.update(self.params.to_dict(ui_json_format=False))
+        self.defaults.pop("workspace", None)
         self.groups_panel = VBox([])
         self.group_auto.observe(self.create_default_groups, names="value")
         self.objects.observe(self.objects_change, names="value")
@@ -110,8 +140,9 @@ class PeakFinder(ObjectDataSelection):
         )
         self.data.observe(self.set_data, names="value")
         self.system.observe(self.set_data, names="value")
-        super().__init__(**kwargs)
+        super().__init__()
         self.pause_refresh = False
+        self.refresh.value = True
         self.previous_line = self.lines.lines.value
         self.smoothing.observe(self.line_update, names="value")
         self.max_migration.observe(self.line_update, names="value")
@@ -131,6 +162,7 @@ class PeakFinder(ObjectDataSelection):
                 "center": self.center,
                 "width": self.width,
                 "plot_trigger": self.plot_trigger,
+                "refresh": self.refresh,
                 "x_label": self.x_label,
             },
         )
@@ -185,25 +217,13 @@ class PeakFinder(ObjectDataSelection):
 
     def __populate__(self, **kwargs):
         super().__populate__(**kwargs)
-        prop_groups = {}
-        count = 0
-        obj_list = self.workspace.get_entity(self.objects.value)
-        for label, params in self.params.groups.items():
-            if params["data"] is not None and any(obj_list):
-                prop_group = [
-                    pg for pg in obj_list[0].property_groups if pg.uid == params["data"]
-                ]
-                if any(prop_group):
-                    count += 1
-                    prop_groups[prop_group[0].name] = {
-                        "data": prop_group[0].uid,
-                        "color": params["color"],
-                        "label": [count],
-                        "properties": prop_group[0].properties,
-                    }
 
-        if len(prop_groups) > 1:
-            self._channel_groups = prop_groups
+        obj_list = self.workspace.get_entity(self.objects.value)
+
+        if any(obj_list) and any(self.params._free_param_dict):
+            self._channel_groups = groups_from_params_dict(
+                obj_list[0], self.params._free_param_dict
+            )
 
         group_list = []
         for pg, params in self._channel_groups.items():
@@ -635,18 +655,9 @@ class PeakFinder(ObjectDataSelection):
     @workspace.setter
     def workspace(self, workspace):
         assert isinstance(workspace, Workspace), f"Workspace must of class {Workspace}"
-        self._workspace = workspace
-        self._h5file = workspace.h5file
-        self.params.input_file.filepath = path.join(
-            path.dirname(self._h5file), self.ga_group_name.value + ".ui.json"
-        )
+        self.base_workspace_changes(workspace)
         self.update_objects_list()
-        self.lines._workspace = workspace
-        self._file_browser.reset(
-            path=self.working_directory,
-            filename=path.basename(self._h5file),
-        )
-        self._file_browser._apply_selection()
+        self.lines.workspace = workspace
 
     @property
     def x_label(self) -> ToggleButtons:
@@ -666,45 +677,45 @@ class PeakFinder(ObjectDataSelection):
         """
         Add a group from dictionary
         """
-        setattr(
-            self,
-            f"Property Group {property_group} Data",
-            Dropdown(
-                description="Group Name:",
-                options=self.data.options,
-            ),
-        )
-        getattr(self, f"Property Group {property_group} Data").name = property_group
-        try:
-            getattr(self, f"Property Group {property_group} Data").value = params[
-                "data"
-            ]
-        except TraitError:
-            pass
-
-        setattr(
-            self,
-            f"Property Group {property_group} Color",
-            ColorPicker(description="Color"),
-        )
-        getattr(self, f"Property Group {property_group} Color").name = property_group
-        try:
-            getattr(self, f"Property Group {property_group} Color").value = str(
-                params["color"]
+        if getattr(self, f"Group {property_group} Data", None) is None:
+            setattr(
+                self,
+                f"Group {property_group} Data",
+                Dropdown(
+                    description="Group Name:",
+                ),
             )
+        widget = getattr(self, f"Group {property_group} Data")
+        widget.name = property_group
+        widget.value = None
+        widget.options = self.data.options
+
+        try:
+            widget.value = params["data"]
+        except TraitError:
+            pass
+        if getattr(self, f"Group {property_group} Color", None) is None:
+            setattr(
+                self,
+                f"Group {property_group} Color",
+                ColorPicker(description="Color"),
+            )
+        getattr(self, f"Group {property_group} Color").name = property_group
+        try:
+            getattr(self, f"Group {property_group} Color").value = str(params["color"])
         except TraitError:
             pass
 
-        getattr(self, f"Property Group {property_group} Data").observe(
+        getattr(self, f"Group {property_group} Data").observe(
             self.edit_group, names="value"
         )
-        getattr(self, f"Property Group {property_group} Color").observe(
+        getattr(self, f"Group {property_group} Color").observe(
             self.edit_group, names="value"
         )
         return VBox(
             [
-                getattr(self, f"Property Group {property_group} Data"),
-                getattr(self, f"Property Group {property_group} Color"),
+                getattr(self, f"Group {property_group} Data"),
+                getattr(self, f"Group {property_group} Color"),
             ],
             layout=Layout(border="solid"),
         )
@@ -713,31 +724,39 @@ class PeakFinder(ObjectDataSelection):
         if self.group_auto.value:
             obj = self.workspace.get_entity(self.objects.value)[0]
             group = [pg for pg in obj.property_groups if pg.uid == self.data.value]
-
             if any(group):
                 channel_groups = self.default_groups_from_property_group(group[0])
-
                 self._channel_groups = channel_groups
                 self.pause_refresh = True
-                for name, group in self._channel_groups.items():
-                    if hasattr(self, f"Property Group {name} Data"):
-                        getattr(self, f"Property Group {name} Data").value = group[
-                            "data"
-                        ]
+                # for name, group in self._channel_groups.items():
+                #     if hasattr(self, f"Template {name} Data"):
+                #         getattr(self, f"Template {name} Data").value = group[
+                #             "data"
+                #         ]
 
+                group_list = []
                 self.update_data_list(None)
+                self.pause_refresh = True
+                for pg, params in self._channel_groups.items():
+                    group_list += [self.add_group_widget(pg, params)]
+
+                self.pause_refresh = False
+                self.groups_panel.children = group_list
+
                 self.set_data(None)
+
         self.group_auto.value = False
+        self._group_auto.button_style = "success"
 
     @staticmethod
-    def default_groups_from_property_group(property_group):
+    def default_groups_from_property_group(property_group, start_index=0):
         parent = property_group.parent
 
         data_list = [
             parent.workspace.get_entity(uid)[0] for uid in property_group.properties
         ]
 
-        start = 0
+        start = start_index
         end = len(data_list)
         block = int((end - start) / 3)
         ranges = {
@@ -761,7 +780,6 @@ class PeakFinder(ObjectDataSelection):
                 "label": [ii + 1],
                 "properties": prop_group.properties,
             }
-            parent.workspace.finalize()
 
         return channel_groups
 
@@ -773,9 +791,7 @@ class PeakFinder(ObjectDataSelection):
         if not self.pause_refresh:
             if isinstance(widget, Dropdown):
                 obj, _ = self.get_selected_entities()
-                group = {
-                    "color": getattr(self, f"Property Group {widget.name} Color").value
-                }
+                group = {"color": getattr(self, f"Group {widget.name} Color").value}
                 if widget.value in [pg.uid for pg in obj.property_groups]:
                     prop_group = [
                         pg for pg in obj.property_groups if pg.uid == widget.value
@@ -830,7 +846,6 @@ class PeakFinder(ObjectDataSelection):
         """
         Re-compute derivatives
         """
-
         if (
             getattr(self, "survey", None) is None
             or len(self.workspace.get_entity(self.lines.data.value)) == 0
@@ -946,6 +961,7 @@ class PeakFinder(ObjectDataSelection):
         center,
         width,
         plot_trigger,
+        refresh,
         x_label,
     ):
         """
@@ -954,16 +970,23 @@ class PeakFinder(ObjectDataSelection):
 
         if (
             self.pause_refresh
-            or getattr(self, "survey", None) is None
+            or not self.refresh.value
             or self.plot_trigger.value is False
-            or len(self.active_channels) == 0
-            or getattr(self.survey, "line_indices", None) is None
-            or len(self.survey.line_indices) < 2
             or not self.plot_result
         ):
             return
+
         self.figure = plt.figure(figsize=(12, 6))
         axs = plt.subplot()
+
+        if (
+            getattr(self, "survey", None) is None
+            or getattr(self.survey, "line_indices", None) is None
+            or len(self.survey.line_indices) < 2
+            or len(self.active_channels) == 0
+        ):
+            return
+
         lims = np.searchsorted(
             self.lines.profile.locations_resampled,
             [
@@ -1154,6 +1177,7 @@ class PeakFinder(ObjectDataSelection):
 
         if (
             self.plot_trigger.value
+            or self.refresh.value
             and hasattr(self.lines, "profile")
             and self.tem_checkbox.value
         ):
@@ -1239,20 +1263,23 @@ class PeakFinder(ObjectDataSelection):
         Observer of :obj:`geoapps.processing.PeakFinder.data`
         Populate the list of available channels and refresh groups
         """
+        self._group_auto.button_style = "warning"
         if getattr(self, "survey", None) is not None and self.data.value is not None:
             self.pause_refresh = True
             self.active_channels = {}
             for group in self.channel_groups.values():
                 for channel in group["properties"]:
                     obj = self.workspace.get_entity(channel)[0]
-                    self.active_channels[channel] = {"name": obj.name}
+
+                    if getattr(obj, "values", None) is not None:
+                        self.active_channels[channel] = {"name": obj.name}
 
             d_min, d_max = np.inf, -np.inf
             thresh_value = np.inf
             if self.tem_checkbox.value:
                 system = self.em_system_specs[self.system.value]
 
-            for uid, params in self.active_channels.items():
+            for uid, params in self.active_channels.copy().items():
                 obj = self.workspace.get_entity(uid)[0]
                 try:
                     if self.tem_checkbox.value:
@@ -1262,15 +1289,27 @@ class PeakFinder(ObjectDataSelection):
                             if ch in params["name"]
                         ]
                         if any(channel):
-                            params["time"] = system["channels"][channel[0]]
+                            self.active_channels[uid]["time"] = system["channels"][
+                                channel[0]
+                            ]
+                        else:
+                            del self.active_channels[uid]
+
+                    self.active_channels[uid]["values"] = (
+                        -1.0
+                    ) ** self.flip_sign.value * obj.values.copy()
+                    thresh_value = np.min(
+                        [
+                            thresh_value,
+                            np.percentile(
+                                np.abs(self.active_channels[uid]["values"]), 95
+                            ),
+                        ]
+                    )
+                    d_min = np.min([d_min, self.active_channels[uid]["values"].min()])
+                    d_max = np.max([d_max, self.active_channels[uid]["values"].max()])
                 except KeyError:
                     continue
-                params["values"] = (-1.0) ** self.flip_sign.value * obj.values.copy()
-                thresh_value = np.min(
-                    [thresh_value, np.percentile(np.abs(params["values"]), 95)]
-                )
-                d_min = np.min([d_min, params["values"].min()])
-                d_max = np.max([d_max, params["values"].max()])
 
             self.pause_refresh = False
             self.plot_trigger.value = False
@@ -1283,45 +1322,43 @@ class PeakFinder(ObjectDataSelection):
             self.line_update(None)
 
     def trigger_click(self, _):
+
+        new_workspace = Workspace(
+            path.join(
+                self.export_directory.selected_path,
+                self._ga_group_name.value + ".geoh5",
+            )
+        )
+        obj, data = self.get_selected_entities()
+
+        new_obj = new_workspace.get_entity(obj.uid)[0]
+        if new_obj is None:
+            obj.copy(parent=new_workspace, copy_children=True)
+
+        self.params.geoh5 = new_workspace.h5file
+        self.params.workspace = new_workspace
+
+        param_dict = {}
         for key, value in self.__dict__.items():
             try:
                 if isinstance(getattr(self, key), Widget):
-                    setattr(self.params, key, getattr(self, key).value)
+                    # setattr(self.params, key, getattr(self, key).value)
+                    param_dict[key] = getattr(self, key).value
             except AttributeError:
                 continue
 
-        self.params._groups = {}
-        self.params.input_file.input_dict = default_ui_json.copy()
-        for group, values in self.channel_groups.items():
-            self.params.input_file.data[f"Property Group {group} Data"] = values["data"]
-            self.params.input_file.data[f"Property Group {group} Color"] = values[
-                "color"
-            ]
-            self.params.groups[group] = {
-                "data": values["data"],
-                "color": values["color"],
-            }
-            self.params.input_file.input_dict[
-                f"Property Group {group} Data"
-            ] = default_ui_json[f"Property Group Data"].copy()
-            self.params.input_file.input_dict[f"Property Group {group} Data"][
-                "group"
-            ] = group
-            self.params.input_file.input_dict[
-                f"Property Group {group} Color"
-            ] = default_ui_json[f"Property Group Color"].copy()
-            self.params.input_file.input_dict[f"Property Group {group} Color"][
-                "group"
-            ] = group
-
-        del self.params.input_file.input_dict[f"Property Group Data"]
-        del self.params.input_file.input_dict[f"Property Group Color"]
-        if "Property Group Data" in self.params.input_file.data.keys():
-            del self.params.input_file.data[f"Property Group Data"]
-            del self.params.input_file.data[f"Property Group Color"]
+        self.params.update(param_dict)
+        self.params.line_field = self.lines.data.value
+        ui_json = deepcopy(self.params.default_ui_json)
         self.params.group_auto = False
-        self.params.write_input_file(name=self.params.ga_group_name)
-        self.run(self.params, output_group=self.ga_group)
+        self.params.write_input_file(
+            ui_json=ui_json,
+            name=path.join(
+                self.export_directory.selected_path,
+                self._ga_group_name.value + ".ui.json",
+            ),
+        )
+        self.run(self.params)
 
     def update_center(self, _):
         """
@@ -1345,6 +1382,12 @@ class PeakFinder(ObjectDataSelection):
         survey = workspace.get_entity(params.objects)[0]
         prop_group = [pg for pg in survey.property_groups if pg.uid == params.data]
 
+        if params.tem_checkbox:
+            system = geophysical_systems.parameters()[params.system]
+            normalization = system["normalization"]
+        else:
+            normalization = [1]
+
         if output_group is None:
             output_group = ContainerGroup.create(
                 workspace, name=string_name(params.ga_group_name)
@@ -1358,23 +1401,8 @@ class PeakFinder(ObjectDataSelection):
                 prop_group[0]
             )
         else:
-            count = 0
-            channel_groups = {}
-            for label, group_params in params.groups.items():
-                if group_params["data"] is not None:
-                    prop_group = [
-                        pg
-                        for pg in survey.property_groups
-                        if pg.uid == group_params["data"]
-                    ]
-                    if any(prop_group):
-                        count += 1
-                        channel_groups[prop_group[0].name] = {
-                            "data": prop_group[0].uid,
-                            "color": group_params["color"],
-                            "label": [count],
-                            "properties": prop_group[0].properties,
-                        }
+
+            channel_groups = groups_from_params_dict(survey, params._free_param_dict)
 
         active_channels = {}
         for group in channel_groups.values():
@@ -1382,12 +1410,6 @@ class PeakFinder(ObjectDataSelection):
                 obj = workspace.get_entity(channel)[0]
                 active_channels[channel] = {"name": obj.name}
 
-        if params.tem_checkbox:
-            system = geophysical_systems.parameters()[params.system]
-            normalization = system["normalization"]
-        else:
-            normalization = [1]
-
         for uid, channel_params in active_channels.items():
             obj = workspace.get_entity(uid)[0]
             if params.tem_checkbox:
@@ -1396,17 +1418,21 @@ class PeakFinder(ObjectDataSelection):
                     channel_params["time"] = system["channels"][channel[0]]
                 else:
                     continue
-            channel_params["values"] = obj.values.copy() * (-1.0) ** params.flip_sign
+            channel_params["values"] = client.scatter(
+                obj.values.copy() * (-1.0) ** params.flip_sign
+            )
 
-        print("Starting Parallel Process...")
+        print("Submitting parallel jobs:")
         anomalies = []
-        for line_id in list(lines):
-            line_indices = line_field.values == line_id
+        locations = client.scatter(survey.vertices.copy())
+
+        for line_id in tqdm(list(lines)):
+            line_indices = np.where(line_field.values == line_id)[0]
 
             anomalies += [
                 client.compute(
                     delayed(find_anomalies)(
-                        survey.vertices,
+                        locations,
                         line_indices,
                         active_channels,
                         channel_groups,
@@ -1421,8 +1447,6 @@ class PeakFinder(ObjectDataSelection):
                     )
                 )
             ]
-
-        all_anomalies = client.gather(anomalies)
         (
             channel_group,
             tau,
@@ -1438,7 +1462,9 @@ class PeakFinder(ObjectDataSelection):
             peaks,
         ) = ([], [], [], [], [], [], [], [], [], [], [], [])
 
-        for line in all_anomalies:
+        print("Processing and collecting results:")
+        for future_line in tqdm(anomalies):
+            line = future_line.result()
             for group in line:
                 if "channel_group" in group.keys() and len(group["cox"]) > 0:
                     channel_group += group["channel_group"]["label"]
@@ -1458,6 +1484,7 @@ class PeakFinder(ObjectDataSelection):
                     skew += [group["skew"]]
                     peaks += [group["peaks"]]
 
+        print("Exporting...")
         if cox:
             channel_group = np.hstack(channel_group)  # Start count at 1
 
@@ -1692,7 +1719,6 @@ def find_anomalies(
         locations=locations[line_indices], smoothing=smoothing, residual=use_residual
     )
     locs = profile.locations_resampled
-
     if data_normalization == "ppm":
         data_normalization = [1e-6]
 
@@ -1969,6 +1995,37 @@ def find_anomalies(
         return groups
 
 
+def groups_from_params_dict(entity: Entity, params_dict: dict):
+    """
+    Generate a dictionary of groups with associate properties from params.
+    """
+    count = 0
+    channel_groups = {}
+    for label, group_params in params_dict.items():
+        if group_params["data"] is not None:
+
+            try:
+                group_id = group_params["data"]
+                if not isinstance(group_id, uuid.UUID):
+                    group_id = uuid.UUID(group_id)
+
+            except ValueError:
+                group_id = None
+
+            prop_group = [pg for pg in entity.property_groups if pg.uid == group_id]
+            if any(prop_group):
+                count += 1
+                channel_groups[prop_group[0].name] = {
+                    "data": prop_group[0].uid,
+                    "color": group_params["color"],
+                    "label": [count],
+                    "properties": prop_group[0].properties,
+                }
+
+    return channel_groups
+
+
 if __name__ == "__main__":
-    params = PeakFinderParams.from_path(sys.argv[1])
+    file = sys.argv[1]
+    params = PeakFinderParams(InputFile(file))
     PeakFinder.run(params)
