@@ -7,6 +7,7 @@
 
 from __future__ import annotations
 
+from collections import OrderedDict
 from uuid import UUID
 
 import ipywidgets as widgets
@@ -30,7 +31,9 @@ class ObjectDataSelection(BaseApplication):
     _objects = None
     _add_groups = False
     _select_multiple = False
-    _object_types = None
+    _object_types = ()
+    _exclusion_types = ()
+    _add_xyz = True
     _find_label = []
 
     def __init__(self, **kwargs):
@@ -39,9 +42,9 @@ class ObjectDataSelection(BaseApplication):
         super().__init__(**kwargs)
 
     @property
-    def add_groups(self):
+    def add_groups(self) -> bool:
         """
-        bool: Add data groups to the list of data choices
+        Add data groups to the list of data choices
         """
         return self._add_groups
 
@@ -50,6 +53,18 @@ class ObjectDataSelection(BaseApplication):
         assert isinstance(value, (bool, str)), "add_groups must be of type bool"
         self._add_groups = value
 
+    @property
+    def add_xyz(self) -> bool:
+        """
+        Add cell or vertices XYZ coordinates in data list
+        """
+        return self._add_xyz
+
+    @add_xyz.setter
+    def add_xyz(self, value):
+        assert isinstance(value, (bool, str)), "add_xyz must be of type bool"
+        self._add_xyz = value
+
     @property
     def data(self) -> Dropdown | SelectMultiple:
         """
@@ -88,7 +103,7 @@ class ObjectDataSelection(BaseApplication):
         """
         :obj:`ipywidgets.VBox`: A box containing all widgets forming the application.
         """
-        self.__populate__(**self.defaults)
+        # self.__populate__(**self.defaults)
         if self._main is None:
             self._main = self.data_panel
             self.update_data_list(None)
@@ -116,22 +131,41 @@ class ObjectDataSelection(BaseApplication):
         """
         Entity type
         """
-        if getattr(self, "_object_types", None) is None:
-            self._object_types = []
-
         return self._object_types
 
     @object_types.setter
     def object_types(self, entity_types):
-        if not isinstance(entity_types, list):
-            entity_types = [entity_types]
+        if not isinstance(entity_types, tuple):
+            entity_types = tuple(entity_types)
 
         for entity_type in entity_types:
             assert issubclass(
                 entity_type, ObjectBase
             ), f"Provided object_types must be instances of {ObjectBase}"
 
-        self._object_types = tuple(entity_types)
+        self._object_types = entity_types
+
+    @property
+    def exclusion_types(self):
+        """
+        Entity type
+        """
+        if getattr(self, "_exclusion_types", None) is None:
+            self._exclusion_types = []
+
+        return self._exclusion_types
+
+    @exclusion_types.setter
+    def exclusion_types(self, entity_types):
+        if not isinstance(entity_types, tuple):
+            entity_types = tuple(entity_types)
+
+        for entity_type in entity_types:
+            assert issubclass(
+                entity_type, ObjectBase
+            ), f"Provided exclusion_types must be instances of {ObjectBase}"
+
+        self._exclusion_types = tuple(entity_types)
 
     @property
     def find_label(self):
@@ -196,8 +230,7 @@ class ObjectDataSelection(BaseApplication):
     @workspace.setter
     def workspace(self, workspace):
         assert isinstance(workspace, Workspace), f"Workspace must of class {Workspace}"
-        self._workspace = workspace
-        self._h5file = workspace.h5file
+        self.base_workspace_changes(workspace)
 
         # Refresh the list of objects
         self.update_objects_list()
@@ -235,6 +268,7 @@ class ObjectDataSelection(BaseApplication):
             return None, None
 
     def update_data_list(self, _):
+        refresh = self.refresh.value
         self.refresh.value = False
         if getattr(self, "_workspace", None) is not None and self._workspace.get_entity(
             self.objects.value
@@ -257,11 +291,20 @@ class ObjectDataSelection(BaseApplication):
 
             if self.add_groups != "only":
                 options += [["--- Channels ---", None]]
-                for child in obj.children:
-                    if isinstance(child, (IntegerData, FloatData)):
-                        options += [[child.name, child.uid]]
+                children_list = {
+                    child.uid: child.name
+                    for child in obj.children
+                    if isinstance(child, (IntegerData, FloatData))
+                }
+                ordered = OrderedDict(sorted(children_list.items(), key=lambda t: t[1]))
+                options += [
+                    [name, uid]
+                    for uid, name in ordered.items()
+                    if "visual parameter" not in name.lower()
+                ]
 
-                options += [["X", "X"], ["Y", "Y"], ["Z", "Z"]]
+                if self.add_xyz:
+                    options += [["X", "X"], ["Y", "Y"], ["Z", "Z"]]
 
             value = self.data.value
             self.data.options = options
@@ -278,7 +321,7 @@ class ObjectDataSelection(BaseApplication):
             self.data.options = []
             self.data.uid_name_map = {}
 
-        self.refresh.value = True
+        self.refresh.value = refresh
 
     def update_objects_list(self):
         if getattr(self, "_workspace", None) is not None:
@@ -293,17 +336,24 @@ class ObjectDataSelection(BaseApplication):
             else:
                 obj_list = self._workspace.objects
 
+            if len(self.exclusion_types) > 0:
+                obj_list = [
+                    obj for obj in obj_list if not isinstance(obj, self.exclusion_types)
+                ]
+
             options = [["", None]] + [
                 [obj.parent.name + "/" + obj.name, obj.uid] for obj in obj_list
             ]
 
-            if value in list(dict(options).values()):  # Silent update
-                self.objects.unobserve(self.update_data_list, names="value")
-                self.objects.options = options
-                self.objects.value = value
-                self._objects.observe(self.update_data_list, names="value")
-            else:
-                self.objects.options = options
+            # if value in list(dict(options).values()):  # Silent update
+            #     self.objects.unobserve(self.update_data_list, names="value")
+            #     self.objects.options = options
+            #     self.objects.value = value
+            #     self._objects.observe(self.update_data_list, names="value")
+            # else:
+            self.objects.options = options
+            # if value in list(dict(options).values()):
+            #     self.objects.value = value
 
     def update_uid_name_map(self):
         """
@@ -325,10 +375,11 @@ class LineOptions(ObjectDataSelection):
 
     defaults = {"find_label": "line"}
     _multiple_lines = None
+    _add_xyz = False
 
     def __init__(self, **kwargs):
 
-        self.defaults = self.update_defaults(**kwargs)
+        self.defaults.update(**kwargs)
 
         super().__init__(**self.defaults)
 
@@ -382,16 +433,21 @@ class LineOptions(ObjectDataSelection):
             else:
                 self.lines.options = [""] + np.unique(data[0].values).tolist()
 
+        else:
+            self.lines.options = [""]
+
 
 class TopographyOptions(ObjectDataSelection):
     """
     Define the topography used by the inversion
     """
 
+    defaults = {}
+
     def __init__(
         self, option_list=["None", "Object", "Relative to Sensor", "Constant"], **kwargs
     ):
-        self.defaults = self.update_defaults(**kwargs)
+        self.defaults.update(**kwargs)
         self.find_label = ["topo", "dem", "dtm", "elevation", "Z"]
         self._offset = FloatText(description="Vertical offset (+ve up)")
         self._constant = FloatText(
