@@ -1,3 +1,10 @@
+#  Copyright (c) 2021 Mira Geoscience Ltd.
+#
+#  This file is part of geoapps.
+#
+#  geoapps is distributed under the terms and conditions of the MIT License
+#  (see LICENSE file at the root of this source code package).
+
 import discretize
 import ipywidgets as widgets
 import matplotlib.pyplot as plt
@@ -8,8 +15,7 @@ from geoh5py.objects import Surface
 from geoh5py.workspace import Workspace
 
 from geoapps.selection import ObjectDataSelection
-from geoapps.simpegPF import PF, Maps, Utils
-from geoapps.utils import RectangularBlock, tensor_2_block_model
+from geoapps.utils.utils import RectangularBlock, tensor_2_block_model
 
 
 def block_model_widget(h5file, inducing_field="50000, 90, 0"):
@@ -236,9 +242,15 @@ def block_model_widget(h5file, inducing_field="50000, 90, 0"):
 
     block_add.observe(block_add_update)
 
-    block_list = widgets.Dropdown(options=[],)
+    block_list = widgets.Dropdown(
+        options=[],
+    )
 
-    block_panel = widgets.VBox([block_list,])
+    block_panel = widgets.VBox(
+        [
+            block_list,
+        ]
+    )
 
     def block_param_update(_):
         if block_list.value is not None:
@@ -262,18 +274,25 @@ def block_model_widget(h5file, inducing_field="50000, 90, 0"):
         },
     )
 
-    sim_name = widgets.Text(value="Simulation_", description="Name:", disabled=False,)
+    sim_name = widgets.Text(
+        value="Simulation_",
+        description="Name:",
+        disabled=False,
+    )
 
     core_cell_size = widgets.Text(
-        value="100, 100, 100", description="Model discretization (m)", disabled=False,
+        value="100, 100, 100",
+        description="Model discretization (m)",
+        disabled=False,
     )
 
-    topography = widgets.Dropdown(options=[None] + names, description="Topography",)
+    topography = widgets.Dropdown(
+        options=[None] + names,
+        description="Topography",
+    )
 
     # Pre-build a list of blocks
-    export_ga = widgets.ToggleButton(
-        value=False, description="Export to GA", icon="check"
-    )
+    export_ga = widgets.ToggleButton(value=False, description="Export", icon="check")
 
     def export_trigger(_):
         if getattr(export_ga, "data", None) is not None and export_ga.value:
@@ -466,10 +485,13 @@ def block_model_widget(h5file, inducing_field="50000, 90, 0"):
         update_data_options("")
 
     survey_type = widgets.Dropdown(
-        options=["Magnetics", "Gravity"], value="Magnetics", description="Survey Type:",
+        options=["Magnetics", "Gravity"],
+        value="Magnetics",
+        description="Survey Type:",
     )
     inducing_field = widgets.Text(
-        value=inducing_field, description="Inducing Field [Amp, Inc, Dec]",
+        value=inducing_field,
+        description="Inducing Field [Amp, Inc, Dec]",
     )
     survey_type_panel = widgets.VBox(
         [survey_type, object_selection.main, components_panel, inducing_field]
@@ -547,7 +569,13 @@ def block_model_widget(h5file, inducing_field="50000, 90, 0"):
             ),
             widgets.HBox(
                 [
-                    widgets.VBox([azimuth, dip, interactive_plot,]),
+                    widgets.VBox(
+                        [
+                            azimuth,
+                            dip,
+                            interactive_plot,
+                        ]
+                    ),
                     widgets.interactive_output(
                         plot_simulation, {"plot": plot_now, "run": forward}
                     ),
