@@ -7,9 +7,11 @@
 
 from .utils import (
     calculate_2D_trend,
+    direct_current_from_simpeg,
     downsample_grid,
     downsample_xy,
     filter_xy,
+    get_inversion_output,
     octree_2_treemesh,
     rotate_xy,
     treemesh_2_octree,
