@@ -11,6 +11,7 @@ import gc
 import json
 import os
 import re
+from uuid import UUID
 
 import dask
 import dask.array as da
@@ -21,12 +22,20 @@ import pandas as pd
 from dask.diagnostics import ProgressBar
 from geoh5py.data import FloatData
 from geoh5py.groups import Group
-from geoh5py.objects import BlockModel, Grid2D, Octree, Surface
+from geoh5py.objects import (
+    BlockModel,
+    CurrentElectrode,
+    Grid2D,
+    Octree,
+    PotentialElectrode,
+    Surface,
+)
 from geoh5py.workspace import Workspace
 from osgeo import gdal
 from scipy.interpolate import interp1d
 from scipy.spatial import ConvexHull, cKDTree
 from shapely.geometry import LineString, mapping
+from SimPEG.electromagnetics.static.resistivity import Survey
 from skimage.measure import marching_cubes
 from sklearn.neighbors import KernelDensity
 
@@ -251,9 +260,14 @@ def geotiff_2_grid(
     assert isinstance(grid, Grid2D), "Parent object must be a Grid2D"
 
     # Replace 0 to nan
-    temp[temp == 0] = np.nan
-    grid.add_data({file_name: {"values": temp.ravel()}})
+    values = temp.ravel()
+    if np.issubdtype(values.dtype, np.integer):
+        values = values.astype("int32")
+        print(values)
+    else:
+        values[values == 0] = np.nan
 
+    grid.add_data({file_name: {"values": values}})
     del tiff_object
     return grid
 
@@ -319,7 +333,9 @@ def export_curve_2_shapefile(
                 c.write(res)
 
 
-def calculate_2D_trend(points, values, order=0, method="all"):
+def calculate_2D_trend(
+    points: np.ndarray, values: np.ndarray, order: int = 0, method: str = "all"
+):
     """
     detrend2D(points, values, order=0, method='all')
 
@@ -328,84 +344,82 @@ def calculate_2D_trend(points, values, order=0, method="all"):
     Parameters:
     ----------
 
-    points: ndarray or floats, shape(npoints, 2)
+    points: array or floats, shape(*, 2)
         Coordinates of input points
 
-    values: ndarray of floats, shape(npoints,)
-        Values to be detrended
+    values: array of floats, shape(*,)
+        Values to be de-trended
 
-    order: int
-        Order of the polynomial to be used
+    order: Order of the polynomial to be used
 
     method: str
         Method to be used for the detrending
             "all": USe all points
-            "corners": Only use points on the convex hull
+            "perimeter": Only use points on the convex hull
 
 
     Returns
     -------
 
-    trend: ndarray of floats, shape(npoints,)
+    trend: array of floats, shape(*,)
         Calculated trend
 
-    coefficients: ndarray of floats, shape(order+1)
+    coefficients: array of floats, shape(order+1)
         Coefficients for the polynomial describing the trend
+
         trend = c[0] + points[:, 0] * c[1] +  points[:, 1] * c[2]
-
     """
+    if not isinstance(order, int) or order < 0:
+        raise ValueError(
+            "Polynomial 'order' should be an integer > 0. "
+            f"Value of {order} provided."
+        )
 
-    assert method in ["all", "corners"], "method must be 'all', or 'corners'"
+    ind_nan = ~np.isnan(values)
+    loc_xy = points[ind_nan, :]
+    values = values[ind_nan]
 
-    assert order in [0, 1, 2], "order must be 0, 1, or 2"
-
-    if method == "corners":
-        hull = ConvexHull(points[:, :2])
+    if method == "perimeter":
+        hull = ConvexHull(loc_xy[:, :2])
         # Extract only those points that make the ConvexHull
-        pts = np.c_[points[hull.vertices, :2], values[hull.vertices]]
-    else:
-        # Extract all points
-        pts = np.c_[points[:, :2], values]
+        loc_xy = loc_xy[hull.vertices, :2]
+        values = values[hull.vertices]
+    elif not method == "all":
+        raise ValueError(
+            "'method' must be either 'all', or 'perimeter'. " f"Value {method} provided"
+        )
 
-    if order == 0:
-        data_trend = np.mean(pts[:, 2]) * np.ones(points[:, 0].shape)
-        print(f"Removed data mean: {data_trend[0]:.6g}")
-        C = np.r_[0, 0, data_trend]
+    # Compute center of mass
+    center_x = np.sum(loc_xy[:, 0] * np.abs(values)) / np.sum(np.abs(values))
+    center_y = np.sum(loc_xy[:, 1] * np.abs(values)) / np.sum(np.abs(values))
 
-    elif order == 1:
-        # best-fit linear plane
-        A = np.c_[pts[:, 0], pts[:, 1], np.ones(pts.shape[0])]
-        C, _, _, _ = np.linalg.lstsq(A, pts[:, 2], rcond=None)  # coefficients
+    polynomial = []
+    xx, yy = np.triu_indices(order + 1)
+    for x, y in zip(xx, yy):
+        polynomial.append(
+            (loc_xy[:, 0] - center_x) ** float(x)
+            * (loc_xy[:, 1] - center_y) ** float(y - x)
+        )
+    polynomial = np.vstack(polynomial).T
 
-        # evaluate at all data locations
-        data_trend = C[0] * points[:, 0] + C[1] * points[:, 1] + C[2]
-        print(f"Removed linear trend with mean: {np.mean(data_trend):.6g}")
+    if polynomial.shape[0] <= polynomial.shape[1]:
+        raise ValueError(
+            "The number of input values must be greater than the number of coefficients in the polynomial. "
+            f"Provided {polynomial.shape[0]} values for a {order}th order polynomial with {polynomial.shape[1]} coefficients."
+        )
 
-    elif order == 2:
-        # best-fit quadratic curve
-        A = np.c_[
-            np.ones(pts.shape[0]),
-            pts[:, :2],
-            np.prod(pts[:, :2], axis=1),
-            pts[:, :2] ** 2,
-        ]
-        C, _, _, _ = np.linalg.lstsq(A, pts[:, 2], rcond=None)
-
-        # evaluate at all data locations
-        data_trend = np.dot(
-            np.c_[
-                np.ones(points[:, 0].shape),
-                points[:, 0],
-                points[:, 1],
-                points[:, 0] * points[:, 1],
-                points[:, 0] ** 2,
-                points[:, 1] ** 2,
-            ],
-            C,
-        ).reshape(points[:, 0].shape)
-
-        print(f"Removed polynomial trend with mean: {np.mean(data_trend):.6g}")
-    return data_trend, C
+    params, _, _, _ = np.linalg.lstsq(polynomial, values, rcond=None)
+    data_trend = np.zeros(points.shape[0])
+    for count, (x, y) in enumerate(zip(xx, yy)):
+        data_trend += (
+            params[count]
+            * (points[:, 0] - center_x) ** float(x)
+            * (points[:, 1] - center_y) ** float(y - x)
+        )
+    print(
+        f"Removed {order}th order polynomial trend with mean: {np.mean(data_trend):.6g}"
+    )
+    return data_trend, params
 
 
 def weighted_average(
@@ -438,23 +452,26 @@ def weighted_average(
         [vals.shape[0] == xyz_in.shape[0] for vals in values]
     ), "Input 'values' must have the same shape as input 'locations'"
 
-    tree = cKDTree(xyz_in)
-    rad, ind = tree.query(xyz_out, n)
-    ind = np.c_[ind]
-    rad = np.c_[rad]
-    rad[rad > max_distance] = np.nan
     avg_values = []
     for value in values:
+        sub = ~np.isnan(value)
+        tree = cKDTree(xyz_in[sub, :])
+        rad, ind = tree.query(xyz_out, n)
+        ind = np.c_[ind]
+        rad = np.c_[rad]
+        rad[rad > max_distance] = np.nan
+
         values_interp = np.zeros(xyz_out.shape[0])
         weight = np.zeros(xyz_out.shape[0])
 
         for ii in range(n):
-            v = value[ind[:, ii]] / (rad[:, ii] + threshold)
+            v = value[sub][ind[:, ii]] / (rad[:, ii] + threshold)
             values_interp = np.nansum([values_interp, v], axis=0)
             w = 1.0 / (rad[:, ii] + threshold)
             weight = np.nansum([weight, w], axis=0)
 
         values_interp[weight > 0] = values_interp[weight > 0] / weight[weight > 0]
+        values_interp[weight == 0] = np.nan
         avg_values += [values_interp]
 
     if return_indices:
@@ -1022,18 +1039,19 @@ def tensor_2_block_model(workspace, mesh, name=None, parent=None, data={}):
     Function to convert a tensor mesh from :obj:`~discretize.TensorMesh` to
     :obj:`~geoh5py.objects.block_model.BlockModel`
     """
+
     block_model = BlockModel.create(
         workspace,
         origin=[mesh.x0[0], mesh.x0[1], mesh.x0[2]],
-        u_cell_delimiters=mesh.vectorNx - mesh.x0[0],
-        v_cell_delimiters=mesh.vectorNy - mesh.x0[1],
+        u_cell_delimiters=(mesh.vectorNx - mesh.x0[0]),
+        v_cell_delimiters=(mesh.vectorNy - mesh.x0[1]),
         z_cell_delimiters=(mesh.vectorNz - mesh.x0[2]),
         name=name,
         parent=parent,
     )
 
     for name, model in data.items():
-        modelMat = mesh.r(model, "CC", "CC", "M")
+        modelMat = mesh.reshape(model, "CC", "CC", "M")
 
         # Transpose the axes
         modelMatT = modelMat.transpose((2, 0, 1))
@@ -1813,6 +1831,44 @@ def iso_surface(
     return surfaces
 
 
+def get_inversion_output(h5file: str | Workspace, inversion_group: str | UUID):
+    """
+    Recover inversion iterations from a ContainerGroup comments.
+    """
+    if isinstance(h5file, Workspace):
+        workspace = h5file
+    else:
+        workspace = Workspace(h5file)
+
+    out = {"time": [], "iteration": [], "phi_d": [], "phi_m": [], "beta": []}
+
+    try:
+        group = workspace.get_entity(inversion_group)[0]
+
+        for comment in group.comments.values:
+            if "Iteration" in comment["Author"]:
+                out["iteration"] += [np.int(comment["Author"].split("_")[1])]
+                out["time"] += [comment["Date"]]
+                values = json.loads(comment["Text"])
+                out["phi_d"] += [float(values["phi_d"])]
+                out["phi_m"] += [float(values["phi_m"])]
+                out["beta"] += [float(values["beta"])]
+
+        if len(out["iteration"]) > 0:
+            out["iteration"] = np.hstack(out["iteration"])
+            ind = np.argsort(out["iteration"])
+            out["iteration"] = out["iteration"][ind]
+            out["phi_d"] = np.hstack(out["phi_d"])[ind]
+            out["phi_m"] = np.hstack(out["phi_m"])[ind]
+            out["time"] = np.hstack(out["time"])[ind]
+
+            return out
+    except IndexError:
+        raise IndexError(
+            f"Inversion group {inversion_group} could not be found in the target geoh5 {h5file}"
+        )
+
+
 def load_json_params(file: str):
     """
     Read input parameters from json
@@ -1830,6 +1886,41 @@ def load_json_params(file: str):
     return params
 
 
+def direct_current_from_simpeg(
+    workspace: Workspace, survey: Survey, name: str = None, data: dict = None
+):
+    """
+    Convert a simpeg direct-current survey to geoh5 format.
+    """
+    u_src_poles, src_pole_id = np.unique(
+        np.r_[survey.locations_a, survey.locations_b], axis=0, return_inverse=True
+    )
+    n_src = int(src_pole_id.shape[0] / 2.0)
+    u_src_cells, src_id = np.unique(
+        np.c_[src_pole_id[:n_src], src_pole_id[n_src:]], axis=0, return_inverse=True
+    )
+    u_rcv_poles, rcv_pole_id = np.unique(
+        np.r_[survey.locations_m, survey.locations_n], axis=0, return_inverse=True
+    )
+    n_rcv = int(rcv_pole_id.shape[0] / 2.0)
+    u_rcv_cells = np.c_[rcv_pole_id[:n_rcv], rcv_pole_id[n_rcv:]]
+    currents = CurrentElectrode.create(
+        workspace, name=name, vertices=u_src_poles, cells=u_src_cells.astype("uint32")
+    )
+    currents.add_default_ab_cell_id()
+
+    potentials = PotentialElectrode.create(
+        workspace, name=name, vertices=u_rcv_poles, cells=u_rcv_cells.astype("uint32")
+    )
+    potentials.current_electrodes = currents
+    potentials.ab_cell_id = np.asarray(src_id + 1, dtype="int32")
+
+    if data is not None:
+        potentials.add_data({key: {"values": value} for key, value in data.items()})
+
+    return currents, potentials
+
+
 colors = [
     "#000000",
     "#FFFF00",
